@@ -23,11 +23,13 @@ module Alexandria
     DESCRIPTION = _('A program to help you manage your book collection.')
     COPYRIGHT = 'Copyright (C) 2004-2005 Laurent Sansonetti'
     AUTHORS = [
+        'Claudio Belotti <bel8@lilik.it>',
         'Constantine Evans <cevans@costinet.org>',
         'Dafydd Harries <daf@muse.19inch.net>',
         'Javier Fernandez-Sanguino Pena <jfs@debian.org>',
         'Kevin Schultz <schultkl@ieee.org>',
         'Laurent Sansonetti <lrz@gnome.org>',
+        'Owain Evans <o.evans@gmail.com>',
         'Pascal Terjan <pterjan@linuxfr.org>',
         'Rene Samselnig <sandman@sdm-net.org>',
         'Robby Stephenson <robby@periapsis.org>',
@@ -40,8 +42,11 @@ module Alexandria
     TRANSLATORS = [ 
         'Dafydd Harries <daf@muse.19inch.net> (cy)',
         'David Weinehall <tao@debian.org> (sv)',
+        'Jiri Pejchal <jiri.pejchal@gmail.com> (cs)',
         'Joachim Breitner <mail@joachim-breitner.de> (de)',
         'Laurent Sansonetti <lrz@gnome.org> (fr)',
+        'Ligia Moreira <ligia.moreira@netvisao.pt> (pt)',
+        'Lucas Rocha <lucasr@im.ufba.br> (pt_BR)',
         'Masao Mutoh <mutoh@highway.ne.jp> (ja)',
         'Miguel Angel Garcia <miguela.garcia3@alu.uclm.es> (es)',
         'Mirko Maischberger <mirko@lilik.it> (it)'
@@ -52,6 +57,7 @@ module Alexandria
     LIST = 'alexandria-list@rubyforge.org'
     BUGREPORT_URL = 'http://rubyforge.org/tracker/?func=add&group_id=205&atid=863'
     WEBSITE_URL = 'http://alexandria.rubyforge.org'
+    DONATE_URL = 'https://www.paypal.com/cgi-bin/webscr?cmd=_xclick&business=lrz%40rubymonks%2eorg&item_name=Alexandria&no_shipping=0&no_note=1&currency_code=EUR'
 
     def self.main
         $DEBUG = !ENV['DEBUG'].nil?
@@ -61,9 +67,20 @@ module Alexandria
     end
 end
 
-require 'alexandria/config'
-require 'alexandria/version'
+unless $MACOSX
+    require 'alexandria/config'
+    require 'alexandria/version'
+else
+    module Alexandria
+        module Config
+            DATA_DIR = OSX::NSBundle.mainBundle.resourcePath.fileSystemRepresentation
+        end
+        VERSION = OSX::NSBundle.mainBundle.infoDictionary.objectForKey('CFBundleVersion').to_s
+    end
+end
+
 require 'alexandria/book'
+require 'alexandria/utils'
 require 'alexandria/library'
 require 'alexandria/execution_queue'
 require 'alexandria/import_library'
@@ -71,4 +88,5 @@ require 'alexandria/export_library'
 require 'alexandria/book_providers'
 require 'alexandria/preferences'
 require 'alexandria/web_themes'
-require 'alexandria/ui'
+
+require $MACOSX ? 'alexandria/macui' : 'alexandria/ui'
