@@ -27,10 +27,18 @@ class BookProviders
         
         def initialize
             super("Amazon")
-            prefs.add("locale", _("Locale site to contact"), "us",
+            prefs.add("locale", _("Locale"), "us",
                        Amazon::Search::LOCALES.keys)
-            prefs.add("dev_token", _("Development token"), "D23XFCO2UKJY82")
-            prefs.add("associate", _("Associate ID"), "calibanorg-20")
+            prefs.add("dev_token", _("Development token"), "142TF8CHT48WYPPS6J82")
+            prefs.add("associate", _("Associate ID"), "calibanorg-20", nil, false)
+            
+            # Backward compatibility hack - the previous developer token has been
+            # revoked.
+            prefs.read
+            token = prefs.variable_named("dev_token")
+            if token and token.value == "D23XFCO2UKJY82"
+                token.new_value = "142TF8CHT48WYPPS6J82"
+            end
         end
 
         def search(criterion, type)
@@ -91,7 +99,7 @@ class BookProviders
                 # the garbled titles to UTF-8. This tries to convert back into
                 # valid UTF-8. It does not always work - see isbn 2259196098
                 # (from the mailing list) for an example.
-                title = GLib.convert(title,'iso-8859-1','utf-8') if req.locale == 'us'
+                title = title.convert('iso-8859-1','utf-8') if req.locale == 'us'
                 book = Book.new(title,
                                 (product.authors.map { |x| x.squeeze(' ') } rescue [ _("n/a") ]),
                                 product.isbn.squeeze(' '),
@@ -104,6 +112,7 @@ class BookProviders
         end
 
         def url(book)
+            return nil unless book.isbn
             url = case prefs["locale"]
                 when "fr"
                     "http://www.amazon.fr/exec/obidos/ASIN/%s"
