@@ -65,6 +65,7 @@ class BookProviders
         end
 
         def url(book)
+            return nil unless book.isbn
             "http://search.barnesandnoble.com/booksearch/isbninquiry.asp?ISBN=" + book.isbn
         end
 
@@ -88,7 +89,7 @@ class BookProviders
             edition = md[1].strip
             raise unless md = /Publisher:[^=]+=\"prodDetailsGen\">([^<]+)/.match(data)
             publisher = md[1].strip
-            raise unless md = /<IMG SRC="(.+\/(\d+|ImageNA).gif)" ALT=("Book Cover"|"Image Not Available") WIDTH="\d+" HEIGHT="\d+" BORDER="0">/.match(data)
+            raise unless md = /<IMG SRC="(.+\/(\d+|ImageNA_product).gif)" ALT=("Book Cover"|"Image Not Available") WIDTH="\d+" HEIGHT="\d+" BORDER="0">/.match(data)
             medium_cover = md[1]
             small_cover = medium_cover.sub(/#{md[2]}/, (md[2].to_i - 1).to_s)
             return [ Book.new(title, authors, isbn, publisher, edition), 
