@@ -32,14 +32,14 @@ class BookProviders
 
         def initialize
             super("Proxis")
-            prefs.add("lang", _("Language of the books to search"), "fr",
+            prefs.add("lang", _("Locale"), "fr",
                       LANGUAGES.keys)
         end
         
         def search(criterion, type)
             prefs.read
 
-            criterion = GLib.convert(criterion, "WINDOWS-1252", "UTF-8")
+            criterion = criterion.convert("windows-1252", "utf-8")
             req = case type
                 when SEARCH_BY_ISBN
                     "p_isbn=#{CGI::escape(criterion)}&p_title=&p_author="
@@ -65,12 +65,17 @@ class BookProviders
                 end
             end
 
+	    # Workaround Proxis returning all editions of a book when searching on ISBN
+	    if type == SEARCH_BY_ISBN
+		    products.delete_if {|n, p| p.first.isbn != criterion}
+	    end
+	    
             raise NoResultsError if products.values.empty?
             type == SEARCH_BY_ISBN ? products.values.first : products.values
         end
         
         def parseBook(product_id)
-            conv = proc { |str| GLib.convert(str, "UTF-8", "WINDOWS-1252") }
+            conv = proc { |str| str.convert("utf-8", "windows-1252") }
             detailspage='http://oas2000.proxis.be/gate/jabba.coreii.g_p?bi=4&sp=DETAILS&mi='+product_id
             product = {}
             product['authors'] = []
@@ -110,8 +115,8 @@ class BookProviders
         end
 
         def url(book)
-            "http://oas2000.proxis.be/gate/jabba.search.submit_search?p_isbn=" + 
-            book.isbn + "&p_item=1"
+            return nil unless book.isbn
+            "http://oas2000.proxis.be/gate/jabba.search.submit_search?p_isbn=" + book.isbn + "&p_item=1"
         end
     end
 end
