@@ -20,8 +20,8 @@ require 'fileutils'
 require 'rexml/document'
 require 'tempfile'
 require 'etc'
-require 'gdk_pixbuf2'
 require 'open-uri'
+require 'observer'
 
 class Array
     def sum
@@ -38,7 +38,10 @@ module Alexandria
         include GetText
         extend GetText
         bindtextdomain(Alexandria::TEXTDOMAIN, nil, nil, "UTF-8")
- 
+
+        BOOK_ADDED, BOOK_UPDATED, BOOK_REMOVED = (0..3).to_a
+        include Observable
+    
         def path
             File.join(DIR, @name)
         end
@@ -48,11 +51,20 @@ module Alexandria
             FileUtils.mkdir_p(library.path) unless File.exists?(library.path)
             Dir.chdir(library.path) do
                 Dir["*" + EXT[:book]].each do |filename|
-                    File.open(filename) do |io|
-                        book = YAML.load(io)
-                        raise "Not a book" unless book.is_a?(Book)
-                        library << book
+                    text = IO.read(filename)
+                    
+                    # Backward compatibility with versions <= 0.6.0, where the 
+                    # loaned_since field was a numeric.
+                    if md = /loaned_since:\s*(\!ruby\/object\:Bignum\s*)?(\d+)\n/.match(text)
+                        new_yaml = Time.at(md[2].to_i).to_yaml
+                        # Remove the "---" prefix.
+                        new_yaml.sub!(/^\s*\-+\s*/, '')
+                        text.sub!(md[0], "loaned_since: #{new_yaml}\n")
                     end
+ 
+                    book = YAML.load(text)
+                    raise "Not a book" unless book.is_a?(Book)
+                    library << book
                 end
 
                 # Since 0.4.0 the cover files '_small.jpg' and 
@@ -86,7 +98,7 @@ module Alexandria
             if a.empty?
                 a << self.load(_("My Library"))
             end
-            a
+            return a
         end
 
         def self.move(source_library, dest_library, *books)
@@ -96,9 +108,17 @@ module Alexandria
                 if File.exists?(source_library.cover(book))
                     FileUtils.mv(source_library.cover(book), dest)
                 end
+
+                source_library.changed
                 source_library.old_delete(book)
+                source_library.notify_observers(source_library, 
+                                                BOOK_REMOVED, 
+                                                book)
+
+                dest_library.changed
                 dest_library.delete_if { |book2| book2.ident == book.ident }
                 dest_library << book
+                dest_library.notify_observers(dest_library, BOOK_ADDED, book)
             end
         end
 
@@ -113,7 +133,7 @@ module Alexandria
         def self.extract_numbers(isbn)
             raise "Nil ISBN" if isbn == nil
 
-            isbn.strip.delete('-').split('').map { |x|
+            isbn.delete('- ').upcase.split('').map { |x|
                 raise InvalidISBNError.new(isbn) unless x =~ /[\dX]/
                 x == 'X' ? 10 : x.to_i
             }
@@ -144,15 +164,50 @@ module Alexandria
         def self.valid_ean?(ean)
             begin
                 numbers = self.extract_numbers(ean)
-                (numbers.length == 13 and self.ean_checksum(numbers) ==
+                (numbers.length == 13 and self.ean_checksum(numbers[0 .. 11]) ==
                     numbers[12]) or
-                (numbers.length == 18 and self.ean_checksum(numbers[0..12]) ==
+                (numbers.length == 18 and self.ean_checksum(numbers[0 .. 11]) ==
                     numbers[12])
             rescue InvalidISBNError
                 false
             end
         end
 
+        def self.upc_checksum(numbers)
+            (10 - ([0, 2, 4, 6, 8, 10].map { |x| numbers[x] }.sum * 3 +
+                  [1, 3, 5, 7, 9].map { |x| numbers[x] }.sum)) % 10
+        end
+
+        def self.valid_upc?(upc)
+            begin
+                numbers = self.extract_numbers(upc)
+                (numbers.length == 17 and self.upc_checksum(numbers[0 .. 10]) ==
+                    numbers[11])
+            rescue InvalidISBNError
+                false
+            end
+        end
+
+	    AMERICAN_UPC_LOOKUP = {
+            "014794" => "08041", "018926" => "0445", "02778" => "0449",
+            "037145" => "0812", "042799" => "0785",  "043144" => "0688",
+            "044903" => "0312", "045863" => "0517", "046594" => "0064",
+            "047132" => "0152", "051487" => "08167", "051488" => "0140",
+            "060771" => "0002", "065373" => "0373", "070992" => "0523",
+            "070993" => "0446", "070999" => "0345", "071001" => "0380",
+            "071009" => "0440", "071125" => "088677", "071136" => "0451",
+            "071149" => "0451", "071152" => "0515", "071162" => "0451",
+            "071268" => "08217", "071831" => "0425", "071842" => "08439",
+            "072742" => "0441", "076714" => "0671", "076783" => "0553",
+            "076814" => "0449", "078021" => "0872", "079808" => "0394",
+            "090129" => "0679", "099455" => "0061", "099769" => "0451"
+            }
+
+        def self.upc_convert(upc)
+            test_upc = upc.map { |x| x.to_s }.join()
+            self.extract_numbers(AMERICAN_UPC_LOOKUP[test_upc])
+        end
+
         def self.canonicalise_isbn(isbn)
             numbers = self.extract_numbers(isbn)
 
@@ -161,9 +216,14 @@ module Alexandria
                 # calculate a checksum. It would be nice if we could validate
                 # the EAN number somehow.
                 numbers[3 .. 11] + [self.isbn_checksum(numbers[3 .. 11])]
+            elsif self.valid_upc?(isbn)
+                # Seems to be a valid UPC number
+                prefix = self.upc_convert(numbers[0 .. 5])
+                isbn_sans_chcksm = prefix + numbers[(8 + prefix.length) .. 17]
+                isbn_sans_chcksm + [self.isbn_checksum(isbn_sans_chcksm)]
             elsif self.valid_isbn?(isbn)
                 # Seems to be a valid ISBN number.
-                numbers[0 .. -2] + [isbn_checksum(numbers[0 .. -2])]
+                numbers[0 .. -2] + [self.isbn_checksum(numbers[0 .. -2])]
             else
                 raise InvalidISBNError.new(isbn)
             end
@@ -172,15 +232,32 @@ module Alexandria
         end
 
         def save(book)
+            changed
+            
+            # Let's initialize the saved identifier if not already
+            # (backward compatibility from 0.4.0).
             book.saved_ident ||= book.ident
+
             if book.ident != book.saved_ident
                 FileUtils.rm(yaml(book.saved_ident))
                 if File.exists?(cover(book.saved_ident))
-                    FileUtils.mv(cover(book.saved_ident), cover(book.ident)) 
+                    FileUtils.mv(cover(book.saved_ident), cover(book.ident))
                 end
+    
+                # Notify before updating the saved identifier, so the views
+                # can still use the old one to update their models.
+                notify_observers(self, BOOK_UPDATED, book)
                 book.saved_ident = book.ident
             end
-            File.open(yaml(book), "w") { |io| io.puts book.to_yaml } 
+            yaml_exists = File.exists?(yaml(book))
+            File.open(yaml(book), "w") { |io| io.puts book.to_yaml }
+            
+            # Do not notify twice.
+            if changed? 
+                notify_observers(self, 
+                                 yaml_exists ? BOOK_UPDATED : BOOK_ADDED, 
+                                 book)
+            end
         end
 
         def save_cover(book, cover_uri)
@@ -199,8 +276,7 @@ module Alexandria
                 end
             
                 # Remove the file if it's blank.
-                pixbuf = Gdk::Pixbuf.new(cover_file)
-                if pixbuf.width == 1 and pixbuf.height == 1
+                if Alexandria::UI::Icons.blank?(cover_file)
                     File.delete(cover_file)
                 end
             end
@@ -212,11 +288,22 @@ module Alexandria
                 # delete the whole library
                 FileUtils.rm_rf(self.path)
             else
+                changed
                 FileUtils.rm_f([yaml(book), cover(book)])
                 old_delete(book)
+                notify_observers(self, BOOK_REMOVED, book)
             end
         end
 
+        alias_method :old_select, :select
+        def select
+            filtered_library = Library.new(@name)
+            self.each do |book|
+                filtered_library << book if yield(book)
+            end
+            return filtered_library
+        end
+
         def cover(something)
             ident = case something
                 when Book
@@ -254,6 +341,10 @@ module Alexandria
             length - n_rated
         end
         
+        def ==(object)
+            object.is_a?(self.class) && object.name == self.name
+        end
+        
         #######
         private
         #######
