@@ -21,6 +21,7 @@ require 'gnome2'
 
 require 'alexandria/ui/icons'
 require 'alexandria/ui/glade_base'
+require 'alexandria/ui/completion_models'
 require 'alexandria/ui/alert_dialog'
 require 'alexandria/ui/about_dialog'
 require 'alexandria/ui/book_properties_dialog_base'
@@ -35,7 +36,7 @@ require 'alexandria/ui/main_app'
 module Alexandria
 module UI
     def self.main
-        Gnome::Program.new(TITLE, VERSION)
+        Gnome::Program.new('alexandria', VERSION).app_datadir = Config::MAIN_DATA_DIR 
         Icons.init
         MainApp.new
         Gtk.main
