@@ -25,12 +25,18 @@ module UI
         def initialize(parent, library, book, &on_close_cb)
             super(parent, library.cover(book))
             @on_close_cb = on_close_cb
-           
+            
             close_button = Gtk::Button.new(Gtk::Stock::CLOSE)
             close_button.signal_connect('pressed') { on_close }
             close_button.show
             @button_box << close_button
-            
+           
+            help_button = Gtk::Button.new(Gtk::Stock::HELP)
+            help_button.signal_connect('pressed') { on_help }
+            help_button.show
+            @button_box << help_button
+            @button_box.set_child_secondary(help_button, true)
+           
             @entry_title.text = @book_properties_dialog.title = book.title
             @entry_isbn.text = book.isbn
             @entry_publisher.text = book.publisher
@@ -52,7 +58,7 @@ module UI
             
             if @checkbutton_loaned.active = book.loaned?
                 @entry_loaned_to.text = (book.loaned_to or "")
-                self.loaned_since = (book.loaned_since or Time.now.tv_sec)
+                self.loaned_since = (book.loaned_since or Time.now)
             end
         end
 
@@ -65,7 +71,7 @@ module UI
                 @book.isbn = nil
             else
                 ary = @library.select { |book| book.ident == @entry_isbn.text }
-                unless ary.empty? or ary == [@book]
+                unless ary.empty? or (ary.length == 1 and ary.first == @book)
                     ErrorDialog.new(@parent, 
                                     _("Couldn't modify the book"), 
                                     _("The EAN/ISBN you provided is already " +
@@ -93,12 +99,20 @@ module UI
            
             @book.loaned = @checkbutton_loaned.active?
             @book.loaned_to = @entry_loaned_to.text
-            @book.loaned_since = @date_loaned_since.time
+            @book.loaned_since = Time.at(@date_loaned_since.time)
            
             @library.save(@book) 
             @on_close_cb.call(@book)
             @book_properties_dialog.destroy
         end
+
+        def on_help
+            begin
+                Gnome::Help.display('alexandria', 'editing-book-properties')
+            rescue 
+                ErrorDialog.new(@preferences_dialog, e.message)
+            end
+        end
     end
 end
 end
