@@ -23,12 +23,17 @@ module UI
         GetText.bindtextdomain(Alexandria::TEXTDOMAIN, nil, nil, "UTF-8")
 
         COVER_MAXWIDTH = 140    # pixels
-        
+
         def initialize(parent, cover_file)
             super('book_properties_dialog.glade')
             @book_properties_dialog.transient_for = parent
             @parent, @cover_file = parent, cover_file
-            
+
+            @entry_title.complete_titles
+            @entry_publisher.complete_publishers
+            @entry_edition.complete_editions
+            @entry_loaned_to.complete_borrowers
+
             @treeview_authors.model = Gtk::ListStore.new(String, TrueClass)
             @treeview_authors.selection.mode = Gtk::SELECTION_SINGLE
             renderer = Gtk::CellRendererText.new
@@ -37,6 +42,10 @@ module UI
                 iter = @treeview_authors.model.get_iter(path)
                 iter[0] = new_text 
             end
+            renderer.signal_connect('editing_started') do |cell, entry, 
+                                                           path_string|
+                entry.complete_authors
+            end
             col = Gtk::TreeViewColumn.new("", renderer, 
                                           :text => 0, 
                                           :editable => 1)
@@ -158,7 +167,7 @@ module UI
         end
 
         def loaned_since=(time)
-            @date_loaned_since.time = time
+            @date_loaned_since.time = time.tv_sec
             # XXX 'date_changed' signal not automatically called after #time=.
             on_loaned_date_changed
         end
