@@ -38,6 +38,12 @@ class Alexandria::Library
     end
 end
 
+class Alexandria::BookProviders::AbstractProvider
+    def action_name
+        "At" + name    
+    end
+end
+
 module Alexandria
 module UI
     class ConflictWhileCopyingDialog < AlertDialog
@@ -62,14 +68,14 @@ module UI
             @response == Gtk::Dialog::RESPONSE_OK
         end
     end
-    
+
     class MainApp < GladeBase 
         include GetText
         GetText.bindtextdomain(Alexandria::TEXTDOMAIN, nil, nil, "UTF-8")
 
         module Columns
             COVER_LIST, COVER_ICON, TITLE, TITLE_REDUCED, AUTHORS,
-                ISBN, PUBLISHER, EDITION, RATING, IDENT = (0..10).to_a
+                ISBN, PUBLISHER, EDITION, RATING, IDENT, NOTES = (0..11).to_a
         end
 
         def initialize
@@ -116,13 +122,26 @@ module UI
                             _("Library '%s' selected") % library.name        
                                 
                         else
-                            s = n_("Library '%s' selected, %d book", 
+                            n_unrated = library.n_unrated
+                            if n_unrated == library.length
+                                n_("Library '%s' selected, %d unrated book",
+                                   "Library '%s' selected, %d unrated books",
+                                   library.length) % [ library.name,
+                                                       library.length ]
+                            elsif n_unrated == 0
+                                n_("Library '%s' selected, %d book", 
                                    "Library '%s' selected, %d books", 
                                    library.length) % [ library.name, 
                                                        library.length ]
-                            s << ", "
-                            s << n_("%d unrated", "%d unrated",
-                                    library.n_unrated) % library.n_unrated
+                            else
+                                n_("Library '%s' selected, %d book, " +
+                                   "%d unrated",
+                                   "Library '%s' selected, %d books, " +
+                                   "%d unrated",
+                                   library.length) % [ library.name, 
+                                                       library.length,
+                                                       n_unrated ]
+                            end
                     end
                 when 1
                     _("'%s' selected") % books.first.title
@@ -139,6 +158,19 @@ module UI
                 @actiongroup["Delete"].sensitive = \
                     @actiongroup["DeselectAll"].sensitive = \
                     @actiongroup["Move"].sensitive = !books.empty?
+                
+                # Sensitize providers URL
+                if books.length == 1
+                    all_url = false
+                    BookProviders.each do |provider|
+                        has_url = provider.url(books.first) != nil
+                        @actiongroup[provider.action_name].sensitive = has_url
+                        all_url = true if has_url and !all_url
+                    end
+                    unless all_url
+                        @actiongroup["OnlineInformation"].sensitive = false
+                    end
+                end
             end
         end
 
@@ -169,10 +201,34 @@ module UI
             @actiongroup["Sidepane"].active = false
         end
 
+        def update(library, kind, book)
+            if library == selected_library
+                @iconview.freeze
+                case kind 
+                    when Library::BOOK_ADDED
+                        append_book(book)
+     
+                    when Library::BOOK_UPDATED
+                        iter = iter_from_ident(book.saved_ident)
+                        fill_iter_with_book(iter, book)
+    
+                    when Library::BOOK_REMOVED
+                        @model.remove(iter_from_book(book))
+                end
+                @iconview.unfreeze
+            end
+        end
+
         #######
         private
         #######
 
+        def display_help
+            Gnome::Help.display('alexandria', 
+                                nil) rescue ErrorDialog.new(@main_app, 
+                                                            e.message)
+        end
+
         def open_web_browser(url)
             unless (cmd = Preferences.instance.www_browser).nil?
                 Thread.new { system(cmd % "\"" + url + "\"") }
@@ -200,30 +256,58 @@ module UI
         end
 
         def load_libraries
+            completion_models = CompletionModels.instance
+            if @libraries
+                @libraries.each do |library| 
+                    library.delete_observer(self)
+                    completion_models.remove_source(library)
+                end
+            end
             @libraries = Library.loadall
+            @libraries.each do |library| 
+                library.add_observer(self)
+                completion_models.add_source(library)
+            end
+        end
+
+        def cache_scaled_icon(icon, width, height)
+            @cache ||= {}
+            @cache[[icon, width, height]] ||= icon.scale(width, height)
         end
 
         ICON_TITLE_MAXLEN = 20   # characters
-        ICON_WIDTH = 48          # pixels
+        ICON_WIDTH = 60
+        ICON_HEIGHT = 90          # pixels
+        REDUCE_TITLE_REGEX = /^(.{#{ICON_TITLE_MAXLEN}}).*$/
         def fill_iter_with_book(iter, book)
-            icon = Icons.cover(selected_library, book)
-            iter[Columns::COVER_LIST] = icon.scale(20, 25)
-            new_height = icon.height / (icon.width / ICON_WIDTH.to_f)
-            iter[Columns::COVER_ICON] = icon.scale(ICON_WIDTH, new_height)
+            iter[Columns::IDENT] = book.ident
             iter[Columns::TITLE] = book.title
-            title = book.title.sub(/^(.{#{ICON_TITLE_MAXLEN}}).*$/, '\1...')
+            title = book.title.sub(REDUCE_TITLE_REGEX, '\1...')
             iter[Columns::TITLE_REDUCED] = title
             iter[Columns::AUTHORS] = book.authors.join(', ')
             iter[Columns::ISBN] = book.isbn
             iter[Columns::PUBLISHER] = book.publisher
             iter[Columns::EDITION] = book.edition
+            iter[Columns::NOTES] = (book.notes or "")
             rating = (book.rating or Book::DEFAULT_RATING)
             iter[Columns::RATING] = 5 - rating # ascending order is the default
-            iter[Columns::IDENT] = book.ident
+
+            icon = Icons.cover(selected_library, book)
+            iter[Columns::COVER_LIST] = cache_scaled_icon(icon, 20, 25)
+
+            if icon.height > ICON_HEIGHT
+                new_width = icon.width / (icon.height / ICON_HEIGHT.to_f)
+                new_height = [ICON_HEIGHT, icon.height].min
+                icon = cache_scaled_icon(icon, new_width, new_height)
+            end
+            if rating == 5
+                icon = icon.tag(Icons::FAVORITE_TAG)
+            end
+            iter[Columns::COVER_ICON] = icon
         end
 
-        def append_book(book)
-            iter = @model.append 
+        def append_book(book, tail=nil)
+            iter = tail ? @model.insert_after(tail) : @model.append
             fill_iter_with_book(iter, book)
             return iter
         end
@@ -268,7 +352,7 @@ module UI
                                                    : Gtk::SORT_ASCENDING)
             @filtered_model.refilter    # force redraw
         end
-        
+
         def setup_books_listview
             # first column
             @listview.model = @listview_model
@@ -281,11 +365,29 @@ module UI
                 cell.pixbuf = iter[Columns::COVER_LIST]
             end        
             renderer = Gtk::CellRendererText.new
-            column.pack_start(renderer, true) 
+=begin
+            # Editable tree views are behaving strangely
+            renderer.signal_connect('editing_started') do |cell, entry, 
+                                                           path_string|
+                entry.complete_titles
+            end
+            renderer.signal_connect('edited') do |cell, path_string, new_string|
+                path = Gtk::TreePath.new(path_string)
+                path = @listview_model.convert_path_to_child_path(path)
+                path = @filtered_model.convert_path_to_child_path(path)
+                iter = @model.get_iter(path)
+                book = book_from_iter(selected_library, iter)
+                book.title = new_string
+                @iconview.freeze
+                fill_iter_with_book(iter, book)
+                @iconview.unfreeze
+            end
+=end
+            column.pack_start(renderer, true)
             column.set_cell_data_func(renderer) do |column, cell, model, iter|
                 iter = @listview_model.convert_iter_to_child_iter(iter)
                 iter = @filtered_model.convert_iter_to_child_iter(iter)
-                cell.text = iter[Columns::TITLE]
+                cell.text, cell.editable = iter[Columns::TITLE], false #true
             end
             column.sort_column_id = Columns::TITLE 
             column.resizable = true
@@ -299,10 +401,11 @@ module UI
                 [ _("Binding"), Columns::EDITION ]
             ]
             names.each do |title, iterid|
-                column = Gtk::TreeViewColumn.new(title, renderer, 
+                renderer = Gtk::CellRendererText.new
+                column = Gtk::TreeViewColumn.new(title, renderer,
                                                  :text => iterid)
-                column.resizable = true
                 column.sort_column_id = iterid
+                column.resizable = true
                 @listview.append_column(column)
             end
 
@@ -344,15 +447,17 @@ module UI
                 cols[i].visible = cols_visibility[i]
             end
         end
-      
+ 
         def refresh_books
             # Clear the views.
             @model.clear
-
+    
             @iconview.freeze
-            selected_library.each { |x| append_book(x) }
+            tail = nil
+            selected_library.each { |book| tail = append_book(book, tail) }
             @filtered_model.refilter
             @iconview.unfreeze
+            @listview.columns_autosize
 
 =begin      
             # Append books - we do that in a separate thread.
@@ -584,12 +689,6 @@ module UI
                                                        library,
                                                        book).replace?
                     end
-                    @iconview.freeze
-                    books.each do |book|
-                        iter = iter_from_book(book)
-                        @model.remove(iter)
-                    end
-                    @iconview.unfreeze
                     Library.move(selected_library,
                                  library, *books)
                 end
@@ -631,11 +730,7 @@ module UI
                 NewBookDialog.new(@main_app, 
                                   @libraries, 
                                   selected_library) do |books, library|
-                    if selected_library == library
-                        @iconview.freeze
-                        books.each { |book| append_book(book) }
-                        @iconview.unfreeze
-                    else
+                    if selected_library != library
                         select_library(library)
                     end
                 end
@@ -643,11 +738,7 @@ module UI
      
             on_add_book_manual = proc do
                 library = selected_library
-                NewBookDialogManual.new(@main_app, library) do |book|
-                    @iconview.freeze
-                    append_book(book)
-                    @iconview.unfreeze
-                end
+                NewBookDialogManual.new(@main_app, library) { |book| }
             end
             
             on_import = proc do 
@@ -665,14 +756,9 @@ module UI
                 books = selected_books
                 if books.length == 1
                     book = books.first
-                    iter = iter_from_book(book)
-                    BookPropertiesDialog.new(@main_app, 
+                    BookPropertiesDialog.new(@main_app,
                                              selected_library, 
-                                             book) do |modified_book|
-                        @iconview.freeze
-                        fill_iter_with_book(iter, modified_book)
-                        @iconview.unfreeze
-                    end
+                                             book) { |modified_book| }
                 end
             end
 
@@ -753,12 +839,7 @@ module UI
                           "selected books from '%s'?") % library.name
                     end
                     if confirm.call(message)
-                        books.each do |book| 
-                            library.delete(book)
-                            @iconview.freeze
-                            @model.remove(iter_from_book(book))
-                            @iconview.unfreeze
-                        end
+                        books.each { |book| library.delete(book) }
                     end
                 end
             end
@@ -777,6 +858,7 @@ module UI
                 end
             end
             on_submit_bug_report = proc { open_web_browser(BUGREPORT_URL) }
+            on_help = proc { display_help }
             on_about = proc { AboutDialog.new(@main_app).show }
 
             standard_actions = [
@@ -805,7 +887,8 @@ module UI
                 ["HelpMenu", nil, _("_Help")],
                 ["SubmitBugReport", Gnome::Stock::MAIL_NEW, _("Submit _Bug Report"), nil, nil,
                  on_submit_bug_report],
-                ["About", Gnome::Stock::ABOUT, _("_About"), nil, nil, on_about],
+                ["Help", Gtk::Stock::HELP, _("Contents"), "F1", nil, on_help],
+                ["About", Gtk::Stock::ABOUT, _("_About"), nil, nil, on_about],
             ]
 
             on_view_sidepane = proc { |ag, a| @paned.child1.visible = a.active? }
@@ -843,7 +926,7 @@ module UI
             ]
 
             providers_actions = BookProviders.map do |provider|
-                ["At" + provider.name, Gtk::Stock::JUMP_TO, 
+                [provider.action_name, Gtk::Stock::JUMP_TO, 
                  _("At _%s") % provider.fullname, nil, nil, 
                  proc { open_web_browser(provider.url(selected_books.first)) }]
             end
@@ -876,7 +959,7 @@ module UI
 
             mid = @uimanager.new_merge_id 
             BookProviders.each do |provider|
-                name = "At" + provider.name    
+                name = provider.action_name    
                 [ "ui/MainMenubar/ViewMenu/OnlineInformation/",
                   "ui/BookPopup/OnlineInformation/",
                   "ui/NoBookPopup/OnlineInformation/" ].each do |path|
@@ -901,9 +984,17 @@ module UI
             @toolbar.insert(-1, Gtk::SeparatorToolItem.new)
            
             cb = Gtk::ComboBox.new
-            [ _("Title contains"), _("Authors contain"), 
-              _("ISBN contains"), _("Publisher contains"), 
+            cb.set_row_separator_func do |model, iter|
+                iter[0] == '-'
+            end
+            [ _("Match everything"),
+              '-',
+              _("Title contains"), 
+              _("Authors contain"),
+              _("ISBN contains"), 
+              _("Publisher contains"),
               _("Notes contain") ].each do |item|
+                
                 cb.append_text(item)
             end
             cb.active = 0
@@ -978,10 +1069,11 @@ module UI
             Gtk::AboutDialog.set_email_hook do |about, link|
                 open_email_client("mailto:" + link)
             end
-            
+ 
+            # The active model. 
             @model = Gtk::ListStore.new(Gdk::Pixbuf, Gdk::Pixbuf, String, 
                                         String, String, String, String, 
-                                        String, Integer, String)
+                                        String, Integer, String, String)
 
             # Filter books according to the search toolbar widgets. 
             @filtered_model = Gtk::TreeModelFilter.new(@model)
@@ -992,13 +1084,17 @@ module UI
                     true
                 else
                     data = case @filter_books_mode
-                        when 0 then iter[Columns::TITLE]
-                        when 1 then iter[Columns::AUTHORS]
-                        when 2 then iter[Columns::ISBN]
-                        when 3 then iter[Columns::PUBLISHER]
-                        when 4 then
-                            book = book_from_iter(selected_library, iter)
-                            ((book != nil and book.notes) or "")
+                        when 0 then 
+                            (iter[Columns::TITLE] or "") +
+                            (iter[Columns::AUTHORS] or "") +
+                            (iter[Columns::ISBN] or "") +
+                            (iter[Columns::PUBLISHER] or "") +
+                            (iter[Columns::NOTES] or "")
+                        when 2 then iter[Columns::TITLE]
+                        when 3 then iter[Columns::AUTHORS]
+                        when 4 then iter[Columns::ISBN]
+                        when 5 then iter[Columns::PUBLISHER]
+                        when 6 then iter[Columns::NOTES]
                     end
                     data != nil and data.downcase.include?(filter.downcase)
                 end     
