@@ -39,8 +39,14 @@ module UI
             add_button.show
             @button_box << add_button
             
+            help_button = Gtk::Button.new(Gtk::Stock::HELP)
+            help_button.signal_connect('pressed') { on_help }
+            help_button.show
+            @button_box << help_button
+            @button_box.set_child_secondary(help_button, true)
+           
             self.rating = Book::DEFAULT_RATING
-            self.cover = Icons::BOOK
+            self.cover = Icons::BOOK_ICON
         end
 
         #######
@@ -91,7 +97,7 @@ module UI
                 book.notes = @textview_notes.buffer.text 
                 book.loaned = @checkbutton_loaned.active?
                 book.loaned_to = @entry_loaned_to.text
-                book.loaned_since = @date_loaned_since.time
+                book.loaned_since = Time.at(@date_loaned_since.time)
 
                 @library << book
                 @library.save(book)
@@ -106,6 +112,14 @@ module UI
                                 e.message)
             end
         end
+        
+        def on_help
+            begin
+                Gnome::Help.display('alexandria', 'add-book-manually')
+            rescue 
+                ErrorDialog.new(@preferences_dialog, e.message)
+            end
+        end
     end
 end
 end
