@@ -1,4 +1,4 @@
-# Copyright (C) 2004 Dafydd Harries
+# Copyright (C) 2004-2005 Dafydd Harries
 #
 # Loosely based on pre-setup.rb from rbbr by Masao Mutoh.
 
@@ -6,6 +6,10 @@ basename = "alexandria"
 config = Config::CONFIG
 podir = srcdir_root + "/po/"
 
+# Extract translations from PO files into other files.
+
+system("intltool-merge -d po alexandria.desktop.in alexandria.desktop")
+
 # Create MO files.
 
 Dir.glob("po/*.po") do |file|
@@ -20,4 +24,6 @@ Dir.glob("po/*.po") do |file|
     end
 
     system("msgfmt po/#{lang}.po -o #{mo_path}/#{basename}.mo")
+
+    raise "msgfmt failed on po/#{lang}.po" if $? != 0
 end
